# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["InexplicitNumberOrderResponse", "OrderingGroup", "OrderingGroupOrder"]


class OrderingGroupOrder(BaseModel):
    number_order_id: str
    """ID of the main number order"""

    sub_number_order_ids: List[str]
    """Array of sub number order IDs"""


class OrderingGroup(BaseModel):
    administrative_area: Optional[str] = None
    """Filter for phone numbers in a given state / province"""

    count_allocated: Optional[int] = None
    """Quantity of phone numbers allocated"""

    count_requested: Optional[int] = None
    """Quantity of phone numbers requested"""

    country_iso: Optional[str] = None
    """Country where you would like to purchase phone numbers"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the ordering group was created"""

    error_reason: Optional[str] = None
    """Error reason if applicable"""

    exclude_held_numbers: Optional[bool] = None
    """
    Filter to exclude phone numbers that are currently on hold/reserved for your
    account.
    """

    national_destination_code: Optional[str] = None
    """Filter by area code"""

    orders: Optional[List[OrderingGroupOrder]] = None
    """Array of orders created to fulfill the inexplicit order"""

    phone_number_type: Optional[str] = None
    """Number type"""

    phone_number_contains: Optional[str] = FieldInfo(alias="phone_number[contains]", default=None)
    """Filter for phone numbers that contain the digits specified"""

    phone_number_ends_with: Optional[str] = FieldInfo(alias="phone_number[ends_with]", default=None)
    """Filter by the ending digits of the phone number"""

    phone_number_starts_with: Optional[str] = FieldInfo(alias="phone_number[starts_with]", default=None)
    """Filter by the starting digits of the phone number"""

    quickship: Optional[bool] = None
    """
    Filter to exclude phone numbers that need additional time after to purchase to
    activate. Only applicable for +1 toll_free numbers.
    """

    status: Optional[Literal["pending", "processing", "failed", "success", "partial_success"]] = None
    """Status of the ordering group"""

    strategy: Optional[Literal["always", "never"]] = None
    """Ordering strategy used"""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the ordering group was updated"""


class InexplicitNumberOrderResponse(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the inexplicit number order"""

    billing_group_id: Optional[str] = None
    """Billing group id to apply to phone numbers that are purchased"""

    connection_id: Optional[str] = None
    """Connection id to apply to phone numbers that are purchased"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created"""

    customer_reference: Optional[str] = None
    """Reference label for the customer"""

    messaging_profile_id: Optional[str] = None
    """Messaging profile id to apply to phone numbers that are purchased"""

    ordering_groups: Optional[List[OrderingGroup]] = None

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["RoomCompositionListParams", "Filter", "FilterDateCreatedAt"]


class RoomCompositionListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[date_created_at][eq], filter[date_created_at][gte],
    filter[date_created_at][lte], filter[session_id], filter[status]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterDateCreatedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room compositions created on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room compositions created on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room compositions created on or before that date."""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[date_created_at][eq], filter[date_created_at][gte], filter[date_created_at][lte], filter[session_id], filter[status]
    """

    date_created_at: FilterDateCreatedAt

    session_id: str
    """The session_id for filtering room compositions."""

    status: Literal["completed", "processing", "enqueued"]
    """The status for filtering room compositions."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["OAuthClientCreateParams"]


class OAuthClientCreateParams(TypedDict, total=False):
    allowed_grant_types: Required[List[Literal["client_credentials", "authorization_code", "refresh_token"]]]
    """List of allowed OAuth grant types"""

    allowed_scopes: Required[SequenceNotStr[str]]
    """List of allowed OAuth scopes"""

    client_type: Required[Literal["public", "confidential"]]
    """OAuth client type"""

    name: Required[str]
    """The name of the OAuth client"""

    logo_uri: str
    """URL of the client logo"""

    policy_uri: str
    """URL of the client's privacy policy"""

    redirect_uris: SequenceNotStr[str]
    """List of redirect URIs (required for authorization_code flow)"""

    require_pkce: bool
    """Whether PKCE (Proof Key for Code Exchange) is required for this client"""

    tos_uri: str
    """URL of the client's terms of service"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    ExternalConnection,
    ExternalConnectionCreateResponse,
    ExternalConnectionDeleteResponse,
    ExternalConnectionUpdateResponse,
    ExternalConnectionRetrieveResponse,
    ExternalConnectionUpdateLocationResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestExternalConnections:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        external_connection = client.external_connections.create(
            external_sip_connection="zoom",
            outbound={},
        )
        assert_matches_type(ExternalConnectionCreateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        external_connection = client.external_connections.create(
            external_sip_connection="zoom",
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "outbound_voice_profile_id",
            },
            active=False,
            inbound={
                "outbound_voice_profile_id": "12345678-1234-1234-1234-123456789012",
                "channel_limit": 10,
            },
            tags=["tag1", "tag2"],
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(ExternalConnectionCreateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.external_connections.with_raw_response.create(
            external_sip_connection="zoom",
            outbound={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = response.parse()
        assert_matches_type(ExternalConnectionCreateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.external_connections.with_streaming_response.create(
            external_sip_connection="zoom",
            outbound={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = response.parse()
            assert_matches_type(ExternalConnectionCreateResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        external_connection = client.external_connections.retrieve(
            "id",
        )
        assert_matches_type(ExternalConnectionRetrieveResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.external_connections.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = response.parse()
        assert_matches_type(ExternalConnectionRetrieveResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.external_connections.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = response.parse()
            assert_matches_type(ExternalConnectionRetrieveResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        external_connection = client.external_connections.update(
            id="id",
            outbound={"outbound_voice_profile_id": "outbound_voice_profile_id"},
        )
        assert_matches_type(ExternalConnectionUpdateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        external_connection = client.external_connections.update(
            id="id",
            outbound={
                "outbound_voice_profile_id": "outbound_voice_profile_id",
                "channel_limit": 10,
            },
            active=False,
            inbound={"channel_limit": 10},
            tags=["tag1", "tag2"],
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(ExternalConnectionUpdateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.external_connections.with_raw_response.update(
            id="id",
            outbound={"outbound_voice_profile_id": "outbound_voice_profile_id"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = response.parse()
        assert_matches_type(ExternalConnectionUpdateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.external_connections.with_streaming_response.update(
            id="id",
            outbound={"outbound_voice_profile_id": "outbound_voice_profile_id"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = response.parse()
            assert_matches_type(ExternalConnectionUpdateResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.with_raw_response.update(
                id="",
                outbound={"outbound_voice_profile_id": "outbound_voice_profile_id"},
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        external_connection = client.external_connections.list()
        assert_matches_type(SyncDefaultFlatPagination[ExternalConnection], external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        external_connection = client.external_connections.list(
            filter={
                "id": "1930241863466354012",
                "connection_name": {"contains": "My Connection"},
                "created_at": "2022-12-31",
                "external_sip_connection": "zoom",
                "phone_number": {"contains": "+15555555555"},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[ExternalConnection], external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.external_connections.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[ExternalConnection], external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.external_connections.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[ExternalConnection], external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        external_connection = client.external_connections.delete(
            "id",
        )
        assert_matches_type(ExternalConnectionDeleteResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.external_connections.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = response.parse()
        assert_matches_type(ExternalConnectionDeleteResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.external_connections.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = response.parse()
            assert_matches_type(ExternalConnectionDeleteResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_location(self, client: Telnyx) -> None:
        external_connection = client.external_connections.update_location(
            location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ExternalConnectionUpdateLocationResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_location(self, client: Telnyx) -> None:
        response = client.external_connections.with_raw_response.update_location(
            location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = response.parse()
        assert_matches_type(ExternalConnectionUpdateLocationResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_location(self, client: Telnyx) -> None:
        with client.external_connections.with_streaming_response.update_location(
            location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = response.parse()
            assert_matches_type(ExternalConnectionUpdateLocationResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update_location(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.external_connections.with_raw_response.update_location(
                location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                id="",
                static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `location_id` but received ''"):
            client.external_connections.with_raw_response.update_location(
                location_id="",
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )


class TestAsyncExternalConnections:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.create(
            external_sip_connection="zoom",
            outbound={},
        )
        assert_matches_type(ExternalConnectionCreateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.create(
            external_sip_connection="zoom",
            outbound={
                "channel_limit": 10,
                "outbound_voice_profile_id": "outbound_voice_profile_id",
            },
            active=False,
            inbound={
                "outbound_voice_profile_id": "12345678-1234-1234-1234-123456789012",
                "channel_limit": 10,
            },
            tags=["tag1", "tag2"],
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(ExternalConnectionCreateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.with_raw_response.create(
            external_sip_connection="zoom",
            outbound={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = await response.parse()
        assert_matches_type(ExternalConnectionCreateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.with_streaming_response.create(
            external_sip_connection="zoom",
            outbound={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = await response.parse()
            assert_matches_type(ExternalConnectionCreateResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.retrieve(
            "id",
        )
        assert_matches_type(ExternalConnectionRetrieveResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = await response.parse()
        assert_matches_type(ExternalConnectionRetrieveResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = await response.parse()
            assert_matches_type(ExternalConnectionRetrieveResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.update(
            id="id",
            outbound={"outbound_voice_profile_id": "outbound_voice_profile_id"},
        )
        assert_matches_type(ExternalConnectionUpdateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.update(
            id="id",
            outbound={
                "outbound_voice_profile_id": "outbound_voice_profile_id",
                "channel_limit": 10,
            },
            active=False,
            inbound={"channel_limit": 10},
            tags=["tag1", "tag2"],
            webhook_event_failover_url="https://failover.example.com",
            webhook_event_url="https://example.com",
            webhook_timeout_secs=25,
        )
        assert_matches_type(ExternalConnectionUpdateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.with_raw_response.update(
            id="id",
            outbound={"outbound_voice_profile_id": "outbound_voice_profile_id"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = await response.parse()
        assert_matches_type(ExternalConnectionUpdateResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.with_streaming_response.update(
            id="id",
            outbound={"outbound_voice_profile_id": "outbound_voice_profile_id"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = await response.parse()
            assert_matches_type(ExternalConnectionUpdateResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.with_raw_response.update(
                id="",
                outbound={"outbound_voice_profile_id": "outbound_voice_profile_id"},
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.list()
        assert_matches_type(AsyncDefaultFlatPagination[ExternalConnection], external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.list(
            filter={
                "id": "1930241863466354012",
                "connection_name": {"contains": "My Connection"},
                "created_at": "2022-12-31",
                "external_sip_connection": "zoom",
                "phone_number": {"contains": "+15555555555"},
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[ExternalConnection], external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[ExternalConnection], external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[ExternalConnection], external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.delete(
            "id",
        )
        assert_matches_type(ExternalConnectionDeleteResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = await response.parse()
        assert_matches_type(ExternalConnectionDeleteResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = await response.parse()
            assert_matches_type(ExternalConnectionDeleteResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_location(self, async_client: AsyncTelnyx) -> None:
        external_connection = await async_client.external_connections.update_location(
            location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(ExternalConnectionUpdateLocationResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_location(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.external_connections.with_raw_response.update_location(
            location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        external_connection = await response.parse()
        assert_matches_type(ExternalConnectionUpdateLocationResponse, external_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_location(self, async_client: AsyncTelnyx) -> None:
        async with async_client.external_connections.with_streaming_response.update_location(
            location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            external_connection = await response.parse()
            assert_matches_type(ExternalConnectionUpdateLocationResponse, external_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update_location(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.external_connections.with_raw_response.update_location(
                location_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                id="",
                static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `location_id` but received ''"):
            await async_client.external_connections.with_raw_response.update_location(
                location_id="",
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                static_emergency_address_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .voice import (
    VoiceResource,
    AsyncVoiceResource,
    VoiceResourceWithRawResponse,
    AsyncVoiceResourceWithRawResponse,
    VoiceResourceWithStreamingResponse,
    AsyncVoiceResourceWithStreamingResponse,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .messaging import (
    MessagingResource,
    AsyncMessagingResource,
    MessagingResourceWithRawResponse,
    AsyncMessagingResourceWithRawResponse,
    MessagingResourceWithStreamingResponse,
    AsyncMessagingResourceWithStreamingResponse,
)
from .voicemail import (
    VoicemailResource,
    AsyncVoicemailResource,
    VoicemailResourceWithRawResponse,
    AsyncVoicemailResourceWithRawResponse,
    VoicemailResourceWithStreamingResponse,
    AsyncVoicemailResourceWithStreamingResponse,
)
from .csv_downloads import (
    CsvDownloadsResource,
    AsyncCsvDownloadsResource,
    CsvDownloadsResourceWithRawResponse,
    AsyncCsvDownloadsResourceWithRawResponse,
    CsvDownloadsResourceWithStreamingResponse,
    AsyncCsvDownloadsResourceWithStreamingResponse,
)
from .phone_numbers import (
    PhoneNumbersResource,
    AsyncPhoneNumbersResource,
    PhoneNumbersResourceWithRawResponse,
    AsyncPhoneNumbersResourceWithRawResponse,
    PhoneNumbersResourceWithStreamingResponse,
    AsyncPhoneNumbersResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "CsvDownloadsResource",
    "AsyncCsvDownloadsResource",
    "CsvDownloadsResourceWithRawResponse",
    "AsyncCsvDownloadsResourceWithRawResponse",
    "CsvDownloadsResourceWithStreamingResponse",
    "AsyncCsvDownloadsResourceWithStreamingResponse",
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "MessagingResource",
    "AsyncMessagingResource",
    "MessagingResourceWithRawResponse",
    "AsyncMessagingResourceWithRawResponse",
    "MessagingResourceWithStreamingResponse",
    "AsyncMessagingResourceWithStreamingResponse",
    "VoiceResource",
    "AsyncVoiceResource",
    "VoiceResourceWithRawResponse",
    "AsyncVoiceResourceWithRawResponse",
    "VoiceResourceWithStreamingResponse",
    "AsyncVoiceResourceWithStreamingResponse",
    "VoicemailResource",
    "AsyncVoicemailResource",
    "VoicemailResourceWithRawResponse",
    "AsyncVoicemailResourceWithRawResponse",
    "VoicemailResourceWithStreamingResponse",
    "AsyncVoicemailResourceWithStreamingResponse",
    "PhoneNumbersResource",
    "AsyncPhoneNumbersResource",
    "PhoneNumbersResourceWithRawResponse",
    "AsyncPhoneNumbersResourceWithRawResponse",
    "PhoneNumbersResourceWithStreamingResponse",
    "AsyncPhoneNumbersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.ai import mcp_server_list_params, mcp_server_create_params, mcp_server_update_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPaginationTopLevelArray, AsyncDefaultFlatPaginationTopLevelArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.ai.mcp_server_list_response import McpServerListResponse
from ...types.ai.mcp_server_create_response import McpServerCreateResponse
from ...types.ai.mcp_server_update_response import McpServerUpdateResponse
from ...types.ai.mcp_server_retrieve_response import McpServerRetrieveResponse

__all__ = ["McpServersResource", "AsyncMcpServersResource"]


class McpServersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> McpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return McpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> McpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return McpServersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        type: str,
        url: str,
        allowed_tools: Optional[SequenceNotStr[str]] | Omit = omit,
        api_key_ref: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        """
        Create a new MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/mcp_servers",
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "url": url,
                    "allowed_tools": allowed_tools,
                    "api_key_ref": api_key_ref,
                },
                mcp_server_create_params.McpServerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServerCreateResponse,
        )

    def retrieve(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerRetrieveResponse:
        """
        Retrieve details for a specific MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return self._get(
            f"/ai/mcp_servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServerRetrieveResponse,
        )

    def update(
        self,
        mcp_server_id: str,
        *,
        id: str | Omit = omit,
        allowed_tools: Optional[SequenceNotStr[str]] | Omit = omit,
        api_key_ref: Optional[str] | Omit = omit,
        created_at: Union[str, datetime] | Omit = omit,
        name: str | Omit = omit,
        type: str | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerUpdateResponse:
        """
        Update an existing MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return self._put(
            f"/ai/mcp_servers/{mcp_server_id}",
            body=maybe_transform(
                {
                    "id": id,
                    "allowed_tools": allowed_tools,
                    "api_key_ref": api_key_ref,
                    "created_at": created_at,
                    "name": name,
                    "type": type,
                    "url": url,
                },
                mcp_server_update_params.McpServerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServerUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        type: str | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPaginationTopLevelArray[McpServerListResponse]:
        """
        Retrieve a list of MCP servers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ai/mcp_servers",
            page=SyncDefaultFlatPaginationTopLevelArray[McpServerListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "type": type,
                        "url": url,
                    },
                    mcp_server_list_params.McpServerListParams,
                ),
            ),
            model=McpServerListResponse,
        )

    def delete(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a specific MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/mcp_servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncMcpServersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMcpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMcpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMcpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMcpServersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        type: str,
        url: str,
        allowed_tools: Optional[SequenceNotStr[str]] | Omit = omit,
        api_key_ref: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerCreateResponse:
        """
        Create a new MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/mcp_servers",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "url": url,
                    "allowed_tools": allowed_tools,
                    "api_key_ref": api_key_ref,
                },
                mcp_server_create_params.McpServerCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServerCreateResponse,
        )

    async def retrieve(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerRetrieveResponse:
        """
        Retrieve details for a specific MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return await self._get(
            f"/ai/mcp_servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServerRetrieveResponse,
        )

    async def update(
        self,
        mcp_server_id: str,
        *,
        id: str | Omit = omit,
        allowed_tools: Optional[SequenceNotStr[str]] | Omit = omit,
        api_key_ref: Optional[str] | Omit = omit,
        created_at: Union[str, datetime] | Omit = omit,
        name: str | Omit = omit,
        type: str | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> McpServerUpdateResponse:
        """
        Update an existing MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        return await self._put(
            f"/ai/mcp_servers/{mcp_server_id}",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "allowed_tools": allowed_tools,
                    "api_key_ref": api_key_ref,
                    "created_at": created_at,
                    "name": name,
                    "type": type,
                    "url": url,
                },
                mcp_server_update_params.McpServerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=McpServerUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        type: str | Omit = omit,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[McpServerListResponse, AsyncDefaultFlatPaginationTopLevelArray[McpServerListResponse]]:
        """
        Retrieve a list of MCP servers.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/ai/mcp_servers",
            page=AsyncDefaultFlatPaginationTopLevelArray[McpServerListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "type": type,
                        "url": url,
                    },
                    mcp_server_list_params.McpServerListParams,
                ),
            ),
            model=McpServerListResponse,
        )

    async def delete(
        self,
        mcp_server_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a specific MCP server.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mcp_server_id:
            raise ValueError(f"Expected a non-empty value for `mcp_server_id` but received {mcp_server_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/mcp_servers/{mcp_server_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class McpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = to_raw_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            mcp_servers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            mcp_servers.update,
        )
        self.list = to_raw_response_wrapper(
            mcp_servers.list,
        )
        self.delete = to_raw_response_wrapper(
            mcp_servers.delete,
        )


class AsyncMcpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = async_to_raw_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            mcp_servers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            mcp_servers.update,
        )
        self.list = async_to_raw_response_wrapper(
            mcp_servers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            mcp_servers.delete,
        )


class McpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = to_streamed_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            mcp_servers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            mcp_servers.update,
        )
        self.list = to_streamed_response_wrapper(
            mcp_servers.list,
        )
        self.delete = to_streamed_response_wrapper(
            mcp_servers.delete,
        )


class AsyncMcpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

        self.create = async_to_streamed_response_wrapper(
            mcp_servers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            mcp_servers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            mcp_servers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            mcp_servers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            mcp_servers.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..ai.assistant_param import AssistantParam
from .aws_voice_settings_param import AwsVoiceSettingsParam
from .transcription_config_param import TranscriptionConfigParam
from .interruption_settings_param import InterruptionSettingsParam
from .telnyx_voice_settings_param import TelnyxVoiceSettingsParam
from .google_transcription_language import GoogleTranscriptionLanguage
from .eleven_labs_voice_settings_param import ElevenLabsVoiceSettingsParam

__all__ = ["ActionGatherUsingAIParams", "MessageHistory", "VoiceSettings"]


class ActionGatherUsingAIParams(TypedDict, total=False):
    parameters: Required[Dict[str, object]]
    """
    The parameters described as a JSON Schema object that needs to be gathered by
    the voice assistant. See the
    [JSON Schema reference](https://json-schema.org/understanding-json-schema) for
    documentation about the format
    """

    assistant: AssistantParam
    """
    Assistant configuration including choice of LLM, custom instructions, and tools.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    greeting: str
    """
    Text that will be played when the gathering starts, if none then nothing will be
    played when the gathering starts. The greeting can be text for any voice or SSML
    for `AWS.Polly.<voice_id>` voices. There is a 3,000 character limit.
    """

    interruption_settings: InterruptionSettingsParam
    """Settings for handling user interruptions during assistant speech"""

    language: GoogleTranscriptionLanguage
    """Language to use for speech recognition"""

    message_history: Iterable[MessageHistory]
    """
    The message history you want the voice assistant to be aware of, this can be
    useful to keep the context of the conversation, or to pass additional
    information to the voice assistant.
    """

    send_message_history_updates: bool
    """Default is `false`.

    If set to `true`, the voice assistant will send updates to the message history
    via the `call.ai_gather.message_history_updated` callback in real time as the
    message history is updated.
    """

    send_partial_results: bool
    """Default is `false`.

    If set to `true`, the voice assistant will send partial results via the
    `call.ai_gather.partial_results` callback in real time as individual fields are
    gathered. If set to `false`, the voice assistant will only send the final result
    via the `call.ai_gather.ended` callback.
    """

    transcription: TranscriptionConfigParam
    """The settings associated with speech to text for the voice assistant.

    This is only relevant if the assistant uses a text-to-text language model. Any
    assistant using a model with native audio support (e.g.
    `fixie-ai/ultravox-v0_4`) will ignore this field.
    """

    user_response_timeout_ms: int
    """
    The number of milliseconds to wait for a user response before the voice
    assistant times out and check if the user is still there.
    """

    voice: str
    """The voice to be used by the voice assistant.

    Currently we support ElevenLabs, Telnyx and AWS voices.

    **Supported Providers:**

    - **AWS:** Use `AWS.Polly.<VoiceId>` (e.g., `AWS.Polly.Joanna`). For neural
      voices, which provide more realistic, human-like speech, append `-Neural` to
      the `VoiceId` (e.g., `AWS.Polly.Joanna-Neural`). Check the
      [available voices](https://docs.aws.amazon.com/polly/latest/dg/available-voices.html)
      for compatibility.
    - **Azure:** Use `Azure.<VoiceId>. (e.g. Azure.en-CA-ClaraNeural,
      Azure.en-CA-LiamNeural, Azure.en-US-BrianMultilingualNeural,
      Azure.en-US-Ava:DragonHDLatestNeural. For a complete list of voices, go to
      [Azure Voice Gallery](https://speech.microsoft.com/portal/voicegallery).)
    - **ElevenLabs:** Use `ElevenLabs.<ModelId>.<VoiceId>` (e.g.,
      `ElevenLabs.BaseModel.John`). The `ModelId` part is optional. To use
      ElevenLabs, you must provide your ElevenLabs API key as an integration secret
      under `"voice_settings": {"api_key_ref": "<secret_id>"}`. See
      [integration secrets documentation](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
      for details. Check
      [available voices](https://elevenlabs.io/docs/api-reference/get-voices).
    - **Telnyx:** Use `Telnyx.<model_id>.<voice_id>`
    """

    voice_settings: VoiceSettings
    """The settings associated with the voice selected"""


class MessageHistory(TypedDict, total=False):
    content: str
    """The content of the message"""

    role: Literal["assistant", "user"]
    """The role of the message sender"""


VoiceSettings: TypeAlias = Union[ElevenLabsVoiceSettingsParam, TelnyxVoiceSettingsParam, AwsVoiceSettingsParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, Annotated, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo
from .._models import set_pydantic_config

__all__ = ["DocumentUploadParams", "Document"]


class DocumentUploadParams(TypedDict, total=False):
    document: Required[Document]


class Document(TypedDict, total=False):
    customer_reference: str
    """A customer reference string for customer look ups."""

    file: Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]
    """
    Alternatively, instead of the URL you can provide the Base64 encoded contents of
    the file you are uploading.
    """

    filename: str
    """The filename of the document."""

    url: str
    """
    If the file is already hosted publicly, you can provide a URL and have the
    documents service fetch it for you.
    """


set_pydantic_config(Document, {"arbitrary_types_allowed": True})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["PortoutDetails"]


class PortoutDetails(BaseModel):
    id: Optional[str] = None

    already_ported: Optional[bool] = None
    """Is true when the number is already ported"""

    authorized_name: Optional[str] = None
    """Name of person authorizing the porting order"""

    carrier_name: Optional[str] = None
    """Carrier the number will be ported out to"""

    city: Optional[str] = None
    """City or municipality of billing address"""

    created_at: Optional[str] = None
    """ISO 8601 formatted date of when the portout was created"""

    current_carrier: Optional[str] = None
    """The current carrier"""

    end_user_name: Optional[str] = None
    """Person name or company name requesting the port"""

    foc_date: Optional[str] = None
    """ISO 8601 formatted Date/Time of the FOC date"""

    host_messaging: Optional[bool] = None
    """
    Indicates whether messaging services should be maintained with Telnyx after the
    port out completes
    """

    inserted_at: Optional[str] = None
    """ISO 8601 formatted date of when the portout was created"""

    lsr: Optional[List[str]] = None
    """The Local Service Request"""

    phone_numbers: Optional[List[str]] = None
    """Phone numbers associated with this portout"""

    pon: Optional[str] = None
    """Port order number assigned by the carrier the number will be ported out to"""

    reason: Optional[str] = None
    """The reason why the order is being rejected by the user.

    If the order is authorized, this field can be left null
    """

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    rejection_code: Optional[int] = None
    """The rejection code for one of the valid rejections to reject a port out order"""

    requested_foc_date: Optional[str] = None
    """ISO 8601 formatted Date/Time of the user requested FOC date"""

    service_address: Optional[str] = None
    """First line of billing address (street address)"""

    spid: Optional[str] = None
    """New service provider spid"""

    state: Optional[str] = None
    """State, province, or similar of billing address"""

    status: Optional[Literal["pending", "authorized", "ported", "rejected", "rejected-pending", "canceled"]] = None
    """Status of portout request"""

    support_key: Optional[str] = None
    """
    A key to reference this port out request when contacting Telnyx customer support
    """

    updated_at: Optional[str] = None
    """ISO 8601 formatted date of when the portout was last updated"""

    user_id: Optional[str] = None
    """Identifies the user (or organization) who requested the port out"""

    vendor: Optional[str] = None
    """Telnyx partner providing network coverage"""

    zip: Optional[str] = None
    """Postal Code of billing address"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ManagedAccountUpdateParams"]


class ManagedAccountUpdateParams(TypedDict, total=False):
    managed_account_allow_custom_pricing: bool
    """
    Boolean value that indicates if the managed account is able to have custom
    pricing set for it or not. If false, uses the pricing of the manager account.
    Defaults to false. This value may be changed, but there may be time lag between
    when the value is changed and pricing changes take effect.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_dtmf_received import CallDtmfReceived

__all__ = ["CallDtmfReceivedWebhookEvent"]


class CallDtmfReceivedWebhookEvent(BaseModel):
    data: Optional[CallDtmfReceived] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["BrandRetrieveSMSOtpStatusResponse"]


class BrandRetrieveSMSOtpStatusResponse(BaseModel):
    """
    Status information for an SMS OTP sent during Sole Proprietor brand verification
    """

    brand_id: str = FieldInfo(alias="brandId")
    """The Brand ID associated with this OTP request"""

    delivery_status: str = FieldInfo(alias="deliveryStatus")
    """The current delivery status of the OTP SMS message.

    Common values include: `DELIVERED_HANDSET`, `PENDING`, `FAILED`, `EXPIRED`
    """

    mobile_phone: str = FieldInfo(alias="mobilePhone")
    """The mobile phone number where the OTP was sent, in E.164 format"""

    reference_id: str = FieldInfo(alias="referenceId")
    """The reference ID for this OTP request, used for status queries"""

    request_date: datetime = FieldInfo(alias="requestDate")
    """The timestamp when the OTP request was initiated"""

    delivery_status_date: Optional[datetime] = FieldInfo(alias="deliveryStatusDate", default=None)
    """The timestamp when the delivery status was last updated"""

    delivery_status_details: Optional[str] = FieldInfo(alias="deliveryStatusDetails", default=None)
    """Additional details about the delivery status"""

    verify_date: Optional[datetime] = FieldInfo(alias="verifyDate", default=None)
    """The timestamp when the OTP was successfully verified (if applicable)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .migration_params import MigrationParams

__all__ = ["MigrationCreateResponse"]


class MigrationCreateResponse(BaseModel):
    data: Optional[MigrationParams] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .porting_associated_phone_number import PortingAssociatedPhoneNumber

__all__ = ["AssociatedPhoneNumberCreateResponse"]


class AssociatedPhoneNumberCreateResponse(BaseModel):
    data: Optional[PortingAssociatedPhoneNumber] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date

from ..._models import BaseModel
from .user_bundle_resource import UserBundleResource
from .billing_bundle_summary import BillingBundleSummary

__all__ = ["UserBundle"]


class UserBundle(BaseModel):
    id: str
    """User bundle's ID, this is used to identify the user bundle in the API."""

    active: bool
    """Status of the user bundle."""

    billing_bundle: BillingBundleSummary

    created_at: date
    """Date the user bundle was created."""

    resources: List[UserBundleResource]

    user_id: str
    """The customer's ID that owns this user bundle."""

    updated_at: Optional[date] = None
    """Date the user bundle was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .phone_number_with_voice_settings import PhoneNumberWithVoiceSettings

__all__ = ["VoiceRetrieveResponse"]


class VoiceRetrieveResponse(BaseModel):
    data: Optional[PhoneNumberWithVoiceSettings] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.types.messages import (
    RcSendResponse,
    RcGenerateDeeplinkResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRcs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_deeplink(self, client: Telnyx) -> None:
        rc = client.messages.rcs.generate_deeplink(
            agent_id="agent_id",
        )
        assert_matches_type(RcGenerateDeeplinkResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_deeplink_with_all_params(self, client: Telnyx) -> None:
        rc = client.messages.rcs.generate_deeplink(
            agent_id="agent_id",
            body="body",
            phone_number="phone_number",
        )
        assert_matches_type(RcGenerateDeeplinkResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_generate_deeplink(self, client: Telnyx) -> None:
        response = client.messages.rcs.with_raw_response.generate_deeplink(
            agent_id="agent_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rc = response.parse()
        assert_matches_type(RcGenerateDeeplinkResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_generate_deeplink(self, client: Telnyx) -> None:
        with client.messages.rcs.with_streaming_response.generate_deeplink(
            agent_id="agent_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rc = response.parse()
            assert_matches_type(RcGenerateDeeplinkResponse, rc, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_generate_deeplink(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.messages.rcs.with_raw_response.generate_deeplink(
                agent_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send(self, client: Telnyx) -> None:
        rc = client.messages.rcs.send(
            agent_id="Agent007",
            agent_message={},
            messaging_profile_id="messaging_profile_id",
            to="+13125551234",
        )
        assert_matches_type(RcSendResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_send_with_all_params(self, client: Telnyx) -> None:
        rc = client.messages.rcs.send(
            agent_id="Agent007",
            agent_message={
                "content_message": {
                    "content_info": {
                        "file_url": "https://example.com/elephant.jpg",
                        "force_refresh": True,
                        "thumbnail_url": "thumbnail_url",
                    },
                    "rich_card": {
                        "carousel_card": {
                            "card_contents": [
                                {
                                    "description": "description",
                                    "media": {
                                        "content_info": {
                                            "file_url": "https://example.com/elephant.jpg",
                                            "force_refresh": True,
                                            "thumbnail_url": "thumbnail_url",
                                        },
                                        "height": "MEDIUM",
                                    },
                                    "suggestions": [
                                        {
                                            "action": {
                                                "create_calendar_event_action": {
                                                    "description": "description",
                                                    "end_time": parse_datetime("2024-10-02T15:02:31Z"),
                                                    "start_time": parse_datetime("2024-10-02T15:01:23Z"),
                                                    "title": "title",
                                                },
                                                "dial_action": {"phone_number": "+13125551234"},
                                                "fallback_url": "fallback_url",
                                                "open_url_action": {
                                                    "application": "BROWSER",
                                                    "url": "http://example.com",
                                                    "webview_view_mode": "HALF",
                                                    "description": "description",
                                                },
                                                "postback_data": "postback_data",
                                                "share_location_action": {"foo": "bar"},
                                                "text": "Hello world",
                                                "view_location_action": {
                                                    "label": "label",
                                                    "lat_long": {
                                                        "latitude": 41.8,
                                                        "longitude": -87.6,
                                                    },
                                                    "query": "query",
                                                },
                                            },
                                            "reply": {
                                                "postback_data": "postback_data",
                                                "text": "text",
                                            },
                                        }
                                    ],
                                    "title": "Elephant",
                                }
                            ],
                            "card_width": "SMALL",
                        },
                        "standalone_card": {
                            "card_content": {
                                "description": "description",
                                "media": {
                                    "content_info": {
                                        "file_url": "https://example.com/elephant.jpg",
                                        "force_refresh": True,
                                        "thumbnail_url": "thumbnail_url",
                                    },
                                    "height": "MEDIUM",
                                },
                                "suggestions": [
                                    {
                                        "action": {
                                            "create_calendar_event_action": {
                                                "description": "description",
                                                "end_time": parse_datetime("2024-10-02T15:02:31Z"),
                                                "start_time": parse_datetime("2024-10-02T15:01:23Z"),
                                                "title": "title",
                                            },
                                            "dial_action": {"phone_number": "+13125551234"},
                                            "fallback_url": "fallback_url",
                                            "open_url_action": {
                                                "application": "BROWSER",
                                                "url": "http://example.com",
                                                "webview_view_mode": "HALF",
                                                "description": "description",
                                            },
                                            "postback_data": "postback_data",
                                            "share_location_action": {"foo": "bar"},
                                            "text": "Hello world",
                                            "view_location_action": {
                                                "label": "label",
                                                "lat_long": {
                                                    "latitude": 41.8,
                                                    "longitude": -87.6,
                                                },
                                                "query": "query",
                                            },
                                        },
                                        "reply": {
                                            "postback_data": "postback_data",
                                            "text": "text",
                                        },
                                    }
                                ],
                                "title": "Elephant",
                            },
                            "card_orientation": "HORIZONTAL",
                            "thumbnail_image_alignment": "LEFT",
                        },
                    },
                    "suggestions": [
                        {
                            "action": {
                                "create_calendar_event_action": {
                                    "description": "description",
                                    "end_time": parse_datetime("2024-10-02T15:02:31Z"),
                                    "start_time": parse_datetime("2024-10-02T15:01:23Z"),
                                    "title": "title",
                                },
                                "dial_action": {"phone_number": "+13125551234"},
                                "fallback_url": "fallback_url",
                                "open_url_action": {
                                    "application": "BROWSER",
                                    "url": "http://example.com",
                                    "webview_view_mode": "HALF",
                                    "description": "description",
                                },
                                "postback_data": "postback_data",
                                "share_location_action": {"foo": "bar"},
                                "text": "Hello world",
                                "view_location_action": {
                                    "label": "label",
                                    "lat_long": {
                                        "latitude": 41.8,
                                        "longitude": -87.6,
                                    },
                                    "query": "query",
                                },
                            },
                            "reply": {
                                "postback_data": "postback_data",
                                "text": "text",
                            },
                        }
                    ],
                    "text": "Hello world!",
                },
                "event": {"event_type": "IS_TYPING"},
                "expire_time": parse_datetime("2024-10-02T15:01:23Z"),
                "ttl": "10.5s",
            },
            messaging_profile_id="messaging_profile_id",
            to="+13125551234",
            mms_fallback={
                "from": "+13125551234",
                "media_urls": ["string"],
                "subject": "Test Message",
                "text": "Hello world!",
            },
            sms_fallback={
                "from": "+13125551234",
                "text": "Hello world!",
            },
            type="RCS",
            webhook_url="webhook_url",
        )
        assert_matches_type(RcSendResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_send(self, client: Telnyx) -> None:
        response = client.messages.rcs.with_raw_response.send(
            agent_id="Agent007",
            agent_message={},
            messaging_profile_id="messaging_profile_id",
            to="+13125551234",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rc = response.parse()
        assert_matches_type(RcSendResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_send(self, client: Telnyx) -> None:
        with client.messages.rcs.with_streaming_response.send(
            agent_id="Agent007",
            agent_message={},
            messaging_profile_id="messaging_profile_id",
            to="+13125551234",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rc = response.parse()
            assert_matches_type(RcSendResponse, rc, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRcs:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_deeplink(self, async_client: AsyncTelnyx) -> None:
        rc = await async_client.messages.rcs.generate_deeplink(
            agent_id="agent_id",
        )
        assert_matches_type(RcGenerateDeeplinkResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_deeplink_with_all_params(self, async_client: AsyncTelnyx) -> None:
        rc = await async_client.messages.rcs.generate_deeplink(
            agent_id="agent_id",
            body="body",
            phone_number="phone_number",
        )
        assert_matches_type(RcGenerateDeeplinkResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_generate_deeplink(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.rcs.with_raw_response.generate_deeplink(
            agent_id="agent_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rc = await response.parse()
        assert_matches_type(RcGenerateDeeplinkResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_generate_deeplink(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.rcs.with_streaming_response.generate_deeplink(
            agent_id="agent_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rc = await response.parse()
            assert_matches_type(RcGenerateDeeplinkResponse, rc, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_generate_deeplink(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.messages.rcs.with_raw_response.generate_deeplink(
                agent_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send(self, async_client: AsyncTelnyx) -> None:
        rc = await async_client.messages.rcs.send(
            agent_id="Agent007",
            agent_message={},
            messaging_profile_id="messaging_profile_id",
            to="+13125551234",
        )
        assert_matches_type(RcSendResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_send_with_all_params(self, async_client: AsyncTelnyx) -> None:
        rc = await async_client.messages.rcs.send(
            agent_id="Agent007",
            agent_message={
                "content_message": {
                    "content_info": {
                        "file_url": "https://example.com/elephant.jpg",
                        "force_refresh": True,
                        "thumbnail_url": "thumbnail_url",
                    },
                    "rich_card": {
                        "carousel_card": {
                            "card_contents": [
                                {
                                    "description": "description",
                                    "media": {
                                        "content_info": {
                                            "file_url": "https://example.com/elephant.jpg",
                                            "force_refresh": True,
                                            "thumbnail_url": "thumbnail_url",
                                        },
                                        "height": "MEDIUM",
                                    },
                                    "suggestions": [
                                        {
                                            "action": {
                                                "create_calendar_event_action": {
                                                    "description": "description",
                                                    "end_time": parse_datetime("2024-10-02T15:02:31Z"),
                                                    "start_time": parse_datetime("2024-10-02T15:01:23Z"),
                                                    "title": "title",
                                                },
                                                "dial_action": {"phone_number": "+13125551234"},
                                                "fallback_url": "fallback_url",
                                                "open_url_action": {
                                                    "application": "BROWSER",
                                                    "url": "http://example.com",
                                                    "webview_view_mode": "HALF",
                                                    "description": "description",
                                                },
                                                "postback_data": "postback_data",
                                                "share_location_action": {"foo": "bar"},
                                                "text": "Hello world",
                                                "view_location_action": {
                                                    "label": "label",
                                                    "lat_long": {
                                                        "latitude": 41.8,
                                                        "longitude": -87.6,
                                                    },
                                                    "query": "query",
                                                },
                                            },
                                            "reply": {
                                                "postback_data": "postback_data",
                                                "text": "text",
                                            },
                                        }
                                    ],
                                    "title": "Elephant",
                                }
                            ],
                            "card_width": "SMALL",
                        },
                        "standalone_card": {
                            "card_content": {
                                "description": "description",
                                "media": {
                                    "content_info": {
                                        "file_url": "https://example.com/elephant.jpg",
                                        "force_refresh": True,
                                        "thumbnail_url": "thumbnail_url",
                                    },
                                    "height": "MEDIUM",
                                },
                                "suggestions": [
                                    {
                                        "action": {
                                            "create_calendar_event_action": {
                                                "description": "description",
                                                "end_time": parse_datetime("2024-10-02T15:02:31Z"),
                                                "start_time": parse_datetime("2024-10-02T15:01:23Z"),
                                                "title": "title",
                                            },
                                            "dial_action": {"phone_number": "+13125551234"},
                                            "fallback_url": "fallback_url",
                                            "open_url_action": {
                                                "application": "BROWSER",
                                                "url": "http://example.com",
                                                "webview_view_mode": "HALF",
                                                "description": "description",
                                            },
                                            "postback_data": "postback_data",
                                            "share_location_action": {"foo": "bar"},
                                            "text": "Hello world",
                                            "view_location_action": {
                                                "label": "label",
                                                "lat_long": {
                                                    "latitude": 41.8,
                                                    "longitude": -87.6,
                                                },
                                                "query": "query",
                                            },
                                        },
                                        "reply": {
                                            "postback_data": "postback_data",
                                            "text": "text",
                                        },
                                    }
                                ],
                                "title": "Elephant",
                            },
                            "card_orientation": "HORIZONTAL",
                            "thumbnail_image_alignment": "LEFT",
                        },
                    },
                    "suggestions": [
                        {
                            "action": {
                                "create_calendar_event_action": {
                                    "description": "description",
                                    "end_time": parse_datetime("2024-10-02T15:02:31Z"),
                                    "start_time": parse_datetime("2024-10-02T15:01:23Z"),
                                    "title": "title",
                                },
                                "dial_action": {"phone_number": "+13125551234"},
                                "fallback_url": "fallback_url",
                                "open_url_action": {
                                    "application": "BROWSER",
                                    "url": "http://example.com",
                                    "webview_view_mode": "HALF",
                                    "description": "description",
                                },
                                "postback_data": "postback_data",
                                "share_location_action": {"foo": "bar"},
                                "text": "Hello world",
                                "view_location_action": {
                                    "label": "label",
                                    "lat_long": {
                                        "latitude": 41.8,
                                        "longitude": -87.6,
                                    },
                                    "query": "query",
                                },
                            },
                            "reply": {
                                "postback_data": "postback_data",
                                "text": "text",
                            },
                        }
                    ],
                    "text": "Hello world!",
                },
                "event": {"event_type": "IS_TYPING"},
                "expire_time": parse_datetime("2024-10-02T15:01:23Z"),
                "ttl": "10.5s",
            },
            messaging_profile_id="messaging_profile_id",
            to="+13125551234",
            mms_fallback={
                "from": "+13125551234",
                "media_urls": ["string"],
                "subject": "Test Message",
                "text": "Hello world!",
            },
            sms_fallback={
                "from": "+13125551234",
                "text": "Hello world!",
            },
            type="RCS",
            webhook_url="webhook_url",
        )
        assert_matches_type(RcSendResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_send(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messages.rcs.with_raw_response.send(
            agent_id="Agent007",
            agent_message={},
            messaging_profile_id="messaging_profile_id",
            to="+13125551234",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rc = await response.parse()
        assert_matches_type(RcSendResponse, rc, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_send(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messages.rcs.with_streaming_response.send(
            agent_id="Agent007",
            agent_message={},
            messaging_profile_id="messaging_profile_id",
            to="+13125551234",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rc = await response.parse()
            assert_matches_type(RcSendResponse, rc, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.messaging_profiles import (
    autoresp_config_list_params,
    autoresp_config_create_params,
    autoresp_config_update_params,
)
from ...types.messaging_profiles.auto_resp_config_response import AutoRespConfigResponse
from ...types.messaging_profiles.autoresp_config_list_response import AutorespConfigListResponse

__all__ = ["AutorespConfigsResource", "AsyncAutorespConfigsResource"]


class AutorespConfigsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AutorespConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AutorespConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AutorespConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AutorespConfigsResourceWithStreamingResponse(self)

    def create(
        self,
        profile_id: str,
        *,
        country_code: str,
        keywords: SequenceNotStr[str],
        op: Literal["start", "stop", "info"],
        resp_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRespConfigResponse:
        """
        Create auto-response setting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return self._post(
            f"/messaging_profiles/{profile_id}/autoresp_configs",
            body=maybe_transform(
                {
                    "country_code": country_code,
                    "keywords": keywords,
                    "op": op,
                    "resp_text": resp_text,
                },
                autoresp_config_create_params.AutorespConfigCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRespConfigResponse,
        )

    def retrieve(
        self,
        autoresp_cfg_id: str,
        *,
        profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRespConfigResponse:
        """
        Get Auto-Response Setting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        if not autoresp_cfg_id:
            raise ValueError(f"Expected a non-empty value for `autoresp_cfg_id` but received {autoresp_cfg_id!r}")
        return self._get(
            f"/messaging_profiles/{profile_id}/autoresp_configs/{autoresp_cfg_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRespConfigResponse,
        )

    def update(
        self,
        autoresp_cfg_id: str,
        *,
        profile_id: str,
        country_code: str,
        keywords: SequenceNotStr[str],
        op: Literal["start", "stop", "info"],
        resp_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRespConfigResponse:
        """
        Update Auto-Response Setting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        if not autoresp_cfg_id:
            raise ValueError(f"Expected a non-empty value for `autoresp_cfg_id` but received {autoresp_cfg_id!r}")
        return self._put(
            f"/messaging_profiles/{profile_id}/autoresp_configs/{autoresp_cfg_id}",
            body=maybe_transform(
                {
                    "country_code": country_code,
                    "keywords": keywords,
                    "op": op,
                    "resp_text": resp_text,
                },
                autoresp_config_update_params.AutorespConfigUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRespConfigResponse,
        )

    def list(
        self,
        profile_id: str,
        *,
        country_code: str | Omit = omit,
        created_at: autoresp_config_list_params.CreatedAt | Omit = omit,
        updated_at: autoresp_config_list_params.UpdatedAt | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutorespConfigListResponse:
        """
        List Auto-Response Settings

        Args:
          created_at:
              Consolidated created_at parameter (deepObject style). Originally:
              created_at[gte], created_at[lte]

          updated_at:
              Consolidated updated_at parameter (deepObject style). Originally:
              updated_at[gte], updated_at[lte]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return self._get(
            f"/messaging_profiles/{profile_id}/autoresp_configs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "country_code": country_code,
                        "created_at": created_at,
                        "updated_at": updated_at,
                    },
                    autoresp_config_list_params.AutorespConfigListParams,
                ),
            ),
            cast_to=AutorespConfigListResponse,
        )

    def delete(
        self,
        autoresp_cfg_id: str,
        *,
        profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Delete Auto-Response Setting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        if not autoresp_cfg_id:
            raise ValueError(f"Expected a non-empty value for `autoresp_cfg_id` but received {autoresp_cfg_id!r}")
        return self._delete(
            f"/messaging_profiles/{profile_id}/autoresp_configs/{autoresp_cfg_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class AsyncAutorespConfigsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAutorespConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAutorespConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAutorespConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAutorespConfigsResourceWithStreamingResponse(self)

    async def create(
        self,
        profile_id: str,
        *,
        country_code: str,
        keywords: SequenceNotStr[str],
        op: Literal["start", "stop", "info"],
        resp_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRespConfigResponse:
        """
        Create auto-response setting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return await self._post(
            f"/messaging_profiles/{profile_id}/autoresp_configs",
            body=await async_maybe_transform(
                {
                    "country_code": country_code,
                    "keywords": keywords,
                    "op": op,
                    "resp_text": resp_text,
                },
                autoresp_config_create_params.AutorespConfigCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRespConfigResponse,
        )

    async def retrieve(
        self,
        autoresp_cfg_id: str,
        *,
        profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRespConfigResponse:
        """
        Get Auto-Response Setting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        if not autoresp_cfg_id:
            raise ValueError(f"Expected a non-empty value for `autoresp_cfg_id` but received {autoresp_cfg_id!r}")
        return await self._get(
            f"/messaging_profiles/{profile_id}/autoresp_configs/{autoresp_cfg_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRespConfigResponse,
        )

    async def update(
        self,
        autoresp_cfg_id: str,
        *,
        profile_id: str,
        country_code: str,
        keywords: SequenceNotStr[str],
        op: Literal["start", "stop", "info"],
        resp_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoRespConfigResponse:
        """
        Update Auto-Response Setting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        if not autoresp_cfg_id:
            raise ValueError(f"Expected a non-empty value for `autoresp_cfg_id` but received {autoresp_cfg_id!r}")
        return await self._put(
            f"/messaging_profiles/{profile_id}/autoresp_configs/{autoresp_cfg_id}",
            body=await async_maybe_transform(
                {
                    "country_code": country_code,
                    "keywords": keywords,
                    "op": op,
                    "resp_text": resp_text,
                },
                autoresp_config_update_params.AutorespConfigUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AutoRespConfigResponse,
        )

    async def list(
        self,
        profile_id: str,
        *,
        country_code: str | Omit = omit,
        created_at: autoresp_config_list_params.CreatedAt | Omit = omit,
        updated_at: autoresp_config_list_params.UpdatedAt | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutorespConfigListResponse:
        """
        List Auto-Response Settings

        Args:
          created_at:
              Consolidated created_at parameter (deepObject style). Originally:
              created_at[gte], created_at[lte]

          updated_at:
              Consolidated updated_at parameter (deepObject style). Originally:
              updated_at[gte], updated_at[lte]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return await self._get(
            f"/messaging_profiles/{profile_id}/autoresp_configs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "country_code": country_code,
                        "created_at": created_at,
                        "updated_at": updated_at,
                    },
                    autoresp_config_list_params.AutorespConfigListParams,
                ),
            ),
            cast_to=AutorespConfigListResponse,
        )

    async def delete(
        self,
        autoresp_cfg_id: str,
        *,
        profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Delete Auto-Response Setting

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        if not autoresp_cfg_id:
            raise ValueError(f"Expected a non-empty value for `autoresp_cfg_id` but received {autoresp_cfg_id!r}")
        return await self._delete(
            f"/messaging_profiles/{profile_id}/autoresp_configs/{autoresp_cfg_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class AutorespConfigsResourceWithRawResponse:
    def __init__(self, autoresp_configs: AutorespConfigsResource) -> None:
        self._autoresp_configs = autoresp_configs

        self.create = to_raw_response_wrapper(
            autoresp_configs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            autoresp_configs.retrieve,
        )
        self.update = to_raw_response_wrapper(
            autoresp_configs.update,
        )
        self.list = to_raw_response_wrapper(
            autoresp_configs.list,
        )
        self.delete = to_raw_response_wrapper(
            autoresp_configs.delete,
        )


class AsyncAutorespConfigsResourceWithRawResponse:
    def __init__(self, autoresp_configs: AsyncAutorespConfigsResource) -> None:
        self._autoresp_configs = autoresp_configs

        self.create = async_to_raw_response_wrapper(
            autoresp_configs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            autoresp_configs.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            autoresp_configs.update,
        )
        self.list = async_to_raw_response_wrapper(
            autoresp_configs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            autoresp_configs.delete,
        )


class AutorespConfigsResourceWithStreamingResponse:
    def __init__(self, autoresp_configs: AutorespConfigsResource) -> None:
        self._autoresp_configs = autoresp_configs

        self.create = to_streamed_response_wrapper(
            autoresp_configs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            autoresp_configs.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            autoresp_configs.update,
        )
        self.list = to_streamed_response_wrapper(
            autoresp_configs.list,
        )
        self.delete = to_streamed_response_wrapper(
            autoresp_configs.delete,
        )


class AsyncAutorespConfigsResourceWithStreamingResponse:
    def __init__(self, autoresp_configs: AsyncAutorespConfigsResource) -> None:
        self._autoresp_configs = autoresp_configs

        self.create = async_to_streamed_response_wrapper(
            autoresp_configs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            autoresp_configs.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            autoresp_configs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            autoresp_configs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            autoresp_configs.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SupportingDocumentCreateResponse", "Data"]


class Data(BaseModel):
    id: str

    created_at: str
    """Supporting document creation timestamp in ISO 8601 format"""

    document_id: str
    """Identifies the associated document"""

    portout_id: str
    """Identifies the associated port request"""

    record_type: str
    """Identifies the type of the resource."""

    type: Literal["loa", "invoice"]
    """Identifies the type of the document"""

    updated_at: str
    """Supporting document last changed timestamp in ISO 8601 format"""


class SupportingDocumentCreateResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import requirement_type_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.requirement_type_list_response import RequirementTypeListResponse
from ..types.requirement_type_retrieve_response import RequirementTypeRetrieveResponse

__all__ = ["RequirementTypesResource", "AsyncRequirementTypesResource"]


class RequirementTypesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RequirementTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RequirementTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RequirementTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RequirementTypesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementTypeRetrieveResponse:
        """
        Retrieve a requirement type by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/requirement_types/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementTypeRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: requirement_type_list_params.Filter | Omit = omit,
        sort: List[Literal["name", "created_at", "updated_at", "-name", "-created_at", "-updated_at"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementTypeListResponse:
        """
        List all requirement types ordered by created_at descending

        Args:
          filter: Consolidated filter parameter for requirement types (deepObject style).
              Originally: filter[name]

          sort: Consolidated sort parameter for requirement types (deepObject style).
              Originally: sort[]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/requirement_types",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "sort": sort,
                    },
                    requirement_type_list_params.RequirementTypeListParams,
                ),
            ),
            cast_to=RequirementTypeListResponse,
        )


class AsyncRequirementTypesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRequirementTypesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRequirementTypesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRequirementTypesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRequirementTypesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementTypeRetrieveResponse:
        """
        Retrieve a requirement type by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/requirement_types/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementTypeRetrieveResponse,
        )

    async def list(
        self,
        *,
        filter: requirement_type_list_params.Filter | Omit = omit,
        sort: List[Literal["name", "created_at", "updated_at", "-name", "-created_at", "-updated_at"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementTypeListResponse:
        """
        List all requirement types ordered by created_at descending

        Args:
          filter: Consolidated filter parameter for requirement types (deepObject style).
              Originally: filter[name]

          sort: Consolidated sort parameter for requirement types (deepObject style).
              Originally: sort[]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/requirement_types",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "sort": sort,
                    },
                    requirement_type_list_params.RequirementTypeListParams,
                ),
            ),
            cast_to=RequirementTypeListResponse,
        )


class RequirementTypesResourceWithRawResponse:
    def __init__(self, requirement_types: RequirementTypesResource) -> None:
        self._requirement_types = requirement_types

        self.retrieve = to_raw_response_wrapper(
            requirement_types.retrieve,
        )
        self.list = to_raw_response_wrapper(
            requirement_types.list,
        )


class AsyncRequirementTypesResourceWithRawResponse:
    def __init__(self, requirement_types: AsyncRequirementTypesResource) -> None:
        self._requirement_types = requirement_types

        self.retrieve = async_to_raw_response_wrapper(
            requirement_types.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            requirement_types.list,
        )


class RequirementTypesResourceWithStreamingResponse:
    def __init__(self, requirement_types: RequirementTypesResource) -> None:
        self._requirement_types = requirement_types

        self.retrieve = to_streamed_response_wrapper(
            requirement_types.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            requirement_types.list,
        )


class AsyncRequirementTypesResourceWithStreamingResponse:
    def __init__(self, requirement_types: AsyncRequirementTypesResource) -> None:
        self._requirement_types = requirement_types

        self.retrieve = async_to_streamed_response_wrapper(
            requirement_types.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            requirement_types.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.ai.assistants import CanaryDeployResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCanaryDeploys:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        canary_deploy = client.ai.assistants.canary_deploys.create(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        )
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.ai.assistants.canary_deploys.with_raw_response.create(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        canary_deploy = response.parse()
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.ai.assistants.canary_deploys.with_streaming_response.create(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            canary_deploy = response.parse()
            assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.canary_deploys.with_raw_response.create(
                assistant_id="",
                versions=[
                    {
                        "percentage": 1,
                        "version_id": "version_id",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        canary_deploy = client.ai.assistants.canary_deploys.retrieve(
            "assistant_id",
        )
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.ai.assistants.canary_deploys.with_raw_response.retrieve(
            "assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        canary_deploy = response.parse()
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.ai.assistants.canary_deploys.with_streaming_response.retrieve(
            "assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            canary_deploy = response.parse()
            assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.canary_deploys.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        canary_deploy = client.ai.assistants.canary_deploys.update(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        )
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.ai.assistants.canary_deploys.with_raw_response.update(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        canary_deploy = response.parse()
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.ai.assistants.canary_deploys.with_streaming_response.update(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            canary_deploy = response.parse()
            assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.canary_deploys.with_raw_response.update(
                assistant_id="",
                versions=[
                    {
                        "percentage": 1,
                        "version_id": "version_id",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        canary_deploy = client.ai.assistants.canary_deploys.delete(
            "assistant_id",
        )
        assert canary_deploy is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.ai.assistants.canary_deploys.with_raw_response.delete(
            "assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        canary_deploy = response.parse()
        assert canary_deploy is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.ai.assistants.canary_deploys.with_streaming_response.delete(
            "assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            canary_deploy = response.parse()
            assert canary_deploy is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            client.ai.assistants.canary_deploys.with_raw_response.delete(
                "",
            )


class TestAsyncCanaryDeploys:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        canary_deploy = await async_client.ai.assistants.canary_deploys.create(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        )
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.canary_deploys.with_raw_response.create(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        canary_deploy = await response.parse()
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.canary_deploys.with_streaming_response.create(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            canary_deploy = await response.parse()
            assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.canary_deploys.with_raw_response.create(
                assistant_id="",
                versions=[
                    {
                        "percentage": 1,
                        "version_id": "version_id",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        canary_deploy = await async_client.ai.assistants.canary_deploys.retrieve(
            "assistant_id",
        )
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.canary_deploys.with_raw_response.retrieve(
            "assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        canary_deploy = await response.parse()
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.canary_deploys.with_streaming_response.retrieve(
            "assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            canary_deploy = await response.parse()
            assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.canary_deploys.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        canary_deploy = await async_client.ai.assistants.canary_deploys.update(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        )
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.canary_deploys.with_raw_response.update(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        canary_deploy = await response.parse()
        assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.canary_deploys.with_streaming_response.update(
            assistant_id="assistant_id",
            versions=[
                {
                    "percentage": 1,
                    "version_id": "version_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            canary_deploy = await response.parse()
            assert_matches_type(CanaryDeployResponse, canary_deploy, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.canary_deploys.with_raw_response.update(
                assistant_id="",
                versions=[
                    {
                        "percentage": 1,
                        "version_id": "version_id",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        canary_deploy = await async_client.ai.assistants.canary_deploys.delete(
            "assistant_id",
        )
        assert canary_deploy is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.assistants.canary_deploys.with_raw_response.delete(
            "assistant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        canary_deploy = await response.parse()
        assert canary_deploy is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.assistants.canary_deploys.with_streaming_response.delete(
            "assistant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            canary_deploy = await response.parse()
            assert canary_deploy is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
            await async_client.ai.assistants.canary_deploys.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .call_calls_params import CallCallsParams as CallCallsParams
from .queue_list_params import QueueListParams as QueueListParams
from .call_update_params import CallUpdateParams as CallUpdateParams
from .call_calls_response import CallCallsResponse as CallCallsResponse
from .queue_create_params import QueueCreateParams as QueueCreateParams
from .queue_list_response import QueueListResponse as QueueListResponse
from .queue_update_params import QueueUpdateParams as QueueUpdateParams
from .call_update_response import CallUpdateResponse as CallUpdateResponse
from .queue_create_response import QueueCreateResponse as QueueCreateResponse
from .queue_update_response import QueueUpdateResponse as QueueUpdateResponse
from .call_retrieve_response import CallRetrieveResponse as CallRetrieveResponse
from .call_siprec_json_params import CallSiprecJsonParams as CallSiprecJsonParams
from .queue_retrieve_response import QueueRetrieveResponse as QueueRetrieveResponse
from .call_streams_json_params import CallStreamsJsonParams as CallStreamsJsonParams
from .conference_update_params import ConferenceUpdateParams as ConferenceUpdateParams
from .call_siprec_json_response import CallSiprecJsonResponse as CallSiprecJsonResponse
from .call_retrieve_calls_params import CallRetrieveCallsParams as CallRetrieveCallsParams
from .call_streams_json_response import CallStreamsJsonResponse as CallStreamsJsonResponse
from .conference_update_response import ConferenceUpdateResponse as ConferenceUpdateResponse
from .call_retrieve_calls_response import CallRetrieveCallsResponse as CallRetrieveCallsResponse
from .conference_retrieve_response import ConferenceRetrieveResponse as ConferenceRetrieveResponse
from .conference_retrieve_conferences_params import (
    ConferenceRetrieveConferencesParams as ConferenceRetrieveConferencesParams,
)
from .conference_retrieve_recordings_response import (
    ConferenceRetrieveRecordingsResponse as ConferenceRetrieveRecordingsResponse,
)
from .conference_retrieve_conferences_response import (
    ConferenceRetrieveConferencesResponse as ConferenceRetrieveConferencesResponse,
)
from .conference_retrieve_recordings_json_response import (
    ConferenceRetrieveRecordingsJsonResponse as ConferenceRetrieveRecordingsJsonResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict
from datetime import datetime

from .._models import BaseModel

__all__ = ["PushCredential"]


class PushCredential(BaseModel):
    id: str
    """Unique identifier of a push credential"""

    alias: str
    """Alias to uniquely identify a credential"""

    certificate: str
    """Apple certificate for sending push notifications. For iOS only"""

    created_at: datetime
    """ISO 8601 timestamp when the room was created"""

    private_key: str
    """Apple private key for a given certificate for sending push notifications.

    For iOS only
    """

    project_account_json_file: Dict[str, object]
    """Google server key for sending push notifications. For Android only"""

    record_type: str

    type: str
    """Type of mobile push credential. Either <code>ios</code> or <code>android</code>"""

    updated_at: datetime
    """ISO 8601 timestamp when the room was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rcs_capabilities import RcsCapabilities as RcsCapabilities
from .rc_invite_test_number_response import RcInviteTestNumberResponse as RcInviteTestNumberResponse
from .rc_list_bulk_capabilities_params import RcListBulkCapabilitiesParams as RcListBulkCapabilitiesParams
from .rc_retrieve_capabilities_response import RcRetrieveCapabilitiesResponse as RcRetrieveCapabilitiesResponse
from .rc_list_bulk_capabilities_response import RcListBulkCapabilitiesResponse as RcListBulkCapabilitiesResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ExternalVettingImportsParams"]


class ExternalVettingImportsParams(TypedDict, total=False):
    evp_id: Required[Annotated[str, PropertyInfo(alias="evpId")]]
    """External vetting provider ID for the brand."""

    vetting_id: Required[Annotated[str, PropertyInfo(alias="vettingId")]]
    """Unique ID that identifies a vetting transaction performed by a vetting provider.

    This ID is provided by the vetting provider at time of vetting.
    """

    vetting_token: Annotated[str, PropertyInfo(alias="vettingToken")]
    """Required by some providers for vetting record confirmation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["MediaResource"]


class MediaResource(BaseModel):
    content_type: Optional[str] = None
    """Content type of the file"""

    created_at: Optional[str] = None
    """ISO 8601 formatted date of when the media resource was created"""

    expires_at: Optional[str] = None
    """ISO 8601 formatted date of when the media resource will expire and be deleted."""

    media_name: Optional[str] = None
    """Uniquely identifies a media resource."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date of when the media resource was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ConsumedData"]


class ConsumedData(BaseModel):
    """Represents the amount of data consumed."""

    amount: Optional[str] = None

    unit: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)
from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.queue_retrieve_response import QueueRetrieveResponse

__all__ = ["QueuesResource", "AsyncQueuesResource"]


class QueuesResource(SyncAPIResource):
    @cached_property
    def calls(self) -> CallsResource:
        return CallsResource(self._client)

    @cached_property
    def with_raw_response(self) -> QueuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return QueuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QueuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return QueuesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        queue_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueRetrieveResponse:
        """
        Retrieve an existing call queue

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        return self._get(
            f"/queues/{queue_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueRetrieveResponse,
        )


class AsyncQueuesResource(AsyncAPIResource):
    @cached_property
    def calls(self) -> AsyncCallsResource:
        return AsyncCallsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncQueuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQueuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQueuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncQueuesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        queue_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueRetrieveResponse:
        """
        Retrieve an existing call queue

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        return await self._get(
            f"/queues/{queue_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueRetrieveResponse,
        )


class QueuesResourceWithRawResponse:
    def __init__(self, queues: QueuesResource) -> None:
        self._queues = queues

        self.retrieve = to_raw_response_wrapper(
            queues.retrieve,
        )

    @cached_property
    def calls(self) -> CallsResourceWithRawResponse:
        return CallsResourceWithRawResponse(self._queues.calls)


class AsyncQueuesResourceWithRawResponse:
    def __init__(self, queues: AsyncQueuesResource) -> None:
        self._queues = queues

        self.retrieve = async_to_raw_response_wrapper(
            queues.retrieve,
        )

    @cached_property
    def calls(self) -> AsyncCallsResourceWithRawResponse:
        return AsyncCallsResourceWithRawResponse(self._queues.calls)


class QueuesResourceWithStreamingResponse:
    def __init__(self, queues: QueuesResource) -> None:
        self._queues = queues

        self.retrieve = to_streamed_response_wrapper(
            queues.retrieve,
        )

    @cached_property
    def calls(self) -> CallsResourceWithStreamingResponse:
        return CallsResourceWithStreamingResponse(self._queues.calls)


class AsyncQueuesResourceWithStreamingResponse:
    def __init__(self, queues: AsyncQueuesResource) -> None:
        self._queues = queues

        self.retrieve = async_to_streamed_response_wrapper(
            queues.retrieve,
        )

    @cached_property
    def calls(self) -> AsyncCallsResourceWithStreamingResponse:
        return AsyncCallsResourceWithStreamingResponse(self._queues.calls)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_playback_started import ConferencePlaybackStarted

__all__ = ["ConferencePlaybackStartedWebhookEvent"]


class ConferencePlaybackStartedWebhookEvent(BaseModel):
    data: Optional[ConferencePlaybackStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ReplacedLinkClick"]


class ReplacedLinkClick(BaseModel):
    message_id: Optional[str] = None
    """The message ID associated with the clicked link."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    time_clicked: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the message request was received."""

    to: Optional[str] = None
    """
    Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
    code).
    """

    url: Optional[str] = None
    """The original link that was sent in the message."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.available_phone_numbers_metadata import AvailablePhoneNumbersMetadata

__all__ = ["AvailablePhoneNumberListResponse", "Data", "DataCostInformation", "DataFeature", "DataRegionInformation"]


class DataCostInformation(BaseModel):
    currency: Optional[str] = None
    """The ISO 4217 code for the currency."""

    monthly_cost: Optional[str] = None

    upfront_cost: Optional[str] = None


class DataFeature(BaseModel):
    name: Optional[str] = None


class DataRegionInformation(BaseModel):
    region_name: Optional[str] = None

    region_type: Optional[Literal["country_code", "rate_center", "state", "location"]] = None


class Data(BaseModel):
    best_effort: Optional[bool] = None
    """
    Specifies whether the phone number is an exact match based on the search
    criteria or not.
    """

    cost_information: Optional[DataCostInformation] = None

    features: Optional[List[DataFeature]] = None

    phone_number: Optional[str] = None

    quickship: Optional[bool] = None
    """
    Specifies whether the phone number can receive calls immediately after purchase
    or not.
    """

    record_type: Optional[Literal["available_phone_number"]] = None

    region_information: Optional[List[DataRegionInformation]] = None

    reservable: Optional[bool] = None
    """Specifies whether the phone number can be reserved before purchase or not."""

    vanity_format: Optional[str] = None


class AvailablePhoneNumberListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[AvailablePhoneNumbersMetadata] = None

    metadata: Optional[AvailablePhoneNumbersMetadata] = None
